{*****************************************************************************
  Copyright: Copyright by yuwei, 2001
  Unit Name: KCommon
  Author:    yuwei
  Email:     william_yuwei@yahoo.com
  Date:      December 23, 2001
  Time:      17:09:18
  Purpose:   All common functions and procedures which used by all Komponents

  History:
  ---------------------------------------------------------------------------
  Date(mm/dd/yy)   Comments
  ---------------------------------------------------------------------------
  12/23/2001       First Create, introduce TKSide, TKSides, TKFrameStyle,
                     beAllSides, TKDeleteItemEvent
                   function KDrawSides, KDrawBevel, KDrawFrame
  12/25/2001       introduct TKMessageLevel
  01/04/2001       Add function KDrawBorderSides
******************************************************************************}

unit KCommon;

interface

{$I KOMPONENTS.INC}

uses Classes, Controls, Graphics,{$IFDEF DELPHI6}Types;{$ELSE}Windows;{$ENDIF}

type
  TKFrameStyle = (fsWindows, fsNone, fsFlat, fsGroove, fsBump, fsLowered,
                  fsRaised);

const
  beAllEdges = [beLeft, beTop, beRight, beBottom];

function KDrawEdges(ACanvas: TCanvas; ABounds: TRect; ULColor, LRColor: TColor;
                    AEdges: TBevelEdges): TRect;

function KDrawBorderEdges(ACanvas: TCanvas; ABounds: TRect;
                          AStyle: TKFrameStyle; AEdges: TBevelEdges): TRect;

implementation

const

  { Frame Style Color constant arrays }

  KULFrameColor: array[TKFrameStyle] of TColor = (clNone,
                                                  clWindow,
                                                  clWindowFrame,
                                                  clBtnShadow,
                                                  clBtnHighlight,
                                                  clBtnShadow,
                                                  clBlack);


  KLRFrameColor: array[TKFrameStyle] of TColor = (clNone,
                                                  clBtnFace,
                                                  clWindowFrame,
                                                  clBtnHighlight,
                                                  clBtnShadow,
                                                  clBtnHighlight,
                                                  clBtnFace);


{-----------------------------------------------------------------------------
  Procedure: KDrawSides
  Author:    yuwei
  Date:      December 23, 2001
  Time:      17:22:42
  Purpose:   Draw a frame with specified the borders on the specified bounds
             of the canvas.
  Arguments:
             ACanvas: TCanvas;
               the canvas where it draws the sides.
             ABounds: TRect;
               the bounds of the canvas for drawing.
             ULColor: TColor;
               the left and top side color.
             LRColor: TColor;
               the right and bottom side color.
             ASides: TBevelEdges;
               which sides it can draw on the canvas.
  Result:
             TRect:
               The bounds within the sides after drawing.
  See also:

  History:
  ---------------------------------------------------------------------------
  Date(mm/dd/yy)   Comments
  ---------------------------------------------------------------------------
  12/23/2001       First Release
-----------------------------------------------------------------------------}
function KDrawEdges(ACanvas: TCanvas; ABounds: TRect; ULColor, LRColor: TColor;
                    AEdges: TBevelEdges): TRect;
begin
  with ACanvas, ABounds do
  begin
    Pen.Style := psSolid;
    Pen.Mode := pmCopy;
    Pen.Color := ULColor;
    if beLeft in AEdges then
    begin
      MoveTo(Left, Top);
      LineTo(Left, Bottom);
    end;
    if beTop in AEdges then
    begin
      MoveTo(Left, Top);
      LineTo(Right, Top);
    end;
    Pen.Color := LRColor;
    if beRight in AEdges then
    begin
      MoveTo(Right - 1, Top);
      LineTo(Right - 1, Bottom);
    end;
    if beBottom in AEdges then
    begin
      MoveTo(Left, Bottom - 1);
      LineTo(Right, Bottom - 1);
    end;
  end;
  if beLeft in AEdges then
    Inc(ABounds.Left);
  if beTop in AEdges then
    Inc(ABounds.Top);
  if beRight in AEdges then
    Dec(ABounds.Right);
  if beBottom in AEdges then
    Dec(ABounds.Bottom);
  Result := ABounds;
end; {KDrawEdges}

function KDrawBorderEdges(ACanvas: TCanvas; ABounds: TRect;
                          AStyle: TKFrameStyle; AEdges: TBevelEdges): TRect;
var
  ULColor, LRColor: TColor;
  R: TRect;
begin
  { Draw the Frame }
  if not (AStyle in [fsNone, fsWindows]) then
  begin
    ULColor := KULFrameColor[AStyle];
    LRColor := KLRFrameColor[AStyle];
    if AStyle in [fsFlat] then
      ABounds := KDrawEdges(ACanvas, ABounds, ULColor, LRColor, AEdges)
    else
    begin
      R := ABounds;
      Inc(R.Left);
      Inc(R.Top);
      KDrawEdges(ACanvas, R, LRColor, LRColor, AEdges);
      OffsetRect(R, -1, -1);
      KDrawEdges(ACanvas, R, ULColor, ULColor, AEdges);
      if beLeft in AEdges then
        Inc(ABounds.Left, 2);
      if beTop in AEdges then
        Inc(ABounds.Top, 2);
      if beRight in AEdges then
        Dec(ABounds.Right, 2);
      if beBottom in AEdges then
        Dec(ABounds.Bottom, 2);
    end;
  end;
  Result := ABounds;
end;

end.
